(function() {
  var Buffer, Cursor, EventEmitter, Point, Region, method, pointMethods, regionMethods, _fn, _fn2, _i, _j, _len, _len2, _ref,
    __hasProp = Object.prototype.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; };

  EventEmitter = require('events').EventEmitter;

  _ref = require('./buffer'), Buffer = _ref.Buffer, Point = _ref.Point, Region = _ref.Region;

  Cursor = (function(_super) {

    __extends(Cursor, _super);

    function Cursor(buffer, row, col) {
      var _this = this;
      this.buffer = buffer;
      this.point = buffer.point(row, col);
      this.point.on("move", function() {
        return _this.emit("move");
      });
      this.buffer.on("line:change", function() {
        var _ref2;
        return (_ref2 = _this.point) != null ? _ref2.round() : void 0;
      });
      this.buffer.on("line:insert", function() {
        var _ref2;
        return (_ref2 = _this.point) != null ? _ref2.round() : void 0;
      });
      this.buffer.on("line:delete", function() {
        var _ref2;
        return (_ref2 = _this.point) != null ? _ref2.round() : void 0;
      });
    }

    Cursor.prototype.serialize = function() {
      if (this.point) {
        return {
          row: this.point.row,
          col: this.point.col
        };
      } else {
        return {
          begin: {
            row: this.region.begin.row,
            col: this.region.begin.col
          },
          end: {
            row: this.region.end.row,
            col: this.region.end.col
          }
        };
      }
    };

    Cursor.prototype.deserialize = function(data) {
      if (data.row != null) {
        return this.moveTo(data.row, data.col);
      } else {
        return this.select(data);
      }
    };

    Cursor.prototype.toPoint = function() {
      var _this = this;
      if (this.point) return;
      this.point = this.region.end;
      this.region = null;
      return this.point.on("move", function() {
        return _this.emit("move");
      });
    };

    Cursor.prototype.toRegion = function() {
      var _this = this;
      if (this.region) return;
      this.region = new Region(this.point, this.point.clone());
      this.point = null;
      this.region.begin.removeAllListeners("move");
      this.region.begin.on("move", function() {
        return _this.emit("move");
      });
      return this.region.end.on("move", function() {
        return _this.emit("move");
      });
    };

    Cursor.prototype.moveTo = function(row, col) {
      this.toPoint();
      this.point.moveTo(row, col);
      return this.point.round();
    };

    Cursor.prototype.select = function(region) {
      this.toRegion();
      this.region.begin.moveTo(region.begin);
      return this.region.end.moveTo(region.end);
    };

    Cursor.prototype.selectTo = function(row, col) {
      this.toRegion();
      this.region.end.moveTo(row, col);
      return this.region.end.round();
    };

    Cursor.prototype.extendTo = function(region) {
      this.toRegion();
      return this.region.extendTo(region);
    };

    Cursor.prototype.moveVertical = function(amount) {
      this.toPoint();
      this.point.moveVertical(amount);
      return this.point.round();
    };

    Cursor.prototype.selectVertical = function(amount) {
      this.toRegion();
      this.region.end.moveVertical(amount);
      return this.region.end.round();
    };

    Cursor.prototype.selectAll = function() {
      var lastCol, lastRow;
      this.toRegion();
      lastRow = this.buffer.lineCount() - 1;
      lastCol = this.buffer.lineLength(lastRow);
      this.region.begin.moveTo(0, 0);
      return this.region.end.moveTo(lastRow, lastCol);
    };

    Cursor.prototype.selectRow = function(row) {
      this.toRegion();
      return this.region.selectRow(row);
    };

    Cursor.prototype.selectRows = function(r1, r2) {
      this.toRegion();
      return this.region.selectRows(r1, r2);
    };

    Cursor.prototype.insert = function(text) {
      this.deleteSelection();
      this.toPoint();
      return this.point.insert(text);
    };

    Cursor.prototype.overwrite = function(text) {
      this.toPoint();
      return this.point.overwrite(text);
    };

    Cursor.prototype.deleteBack = function() {
      if (this.deleteSelection()) return;
      this.toPoint();
      return this.point.deleteBack();
    };

    Cursor.prototype.deleteForward = function() {
      if (this.deleteSelection()) return;
      this.toPoint();
      return this.point.deleteForward();
    };

    Cursor.prototype.deleteWordBack = function() {
      this.deleteSelection();
      this.toPoint();
      return this.point.deleteWordBack();
    };

    Cursor.prototype.deleteWordForward = function() {
      this.deleteSelection();
      this.toPoint();
      return this.point.deleteWordForward();
    };

    Cursor.prototype.newLine = function() {
      this.toPoint();
      return this.point.newLine();
    };

    Cursor.prototype.deleteSelection = function() {
      if (!this.region || this.region.isEmpty()) {
        return false;
      } else {
        this.region["delete"]();
        this.emit("move");
        return true;
      }
    };

    Cursor.prototype.deleteRows = function() {
      var begin, end, _ref2;
      this.toRegion();
      _ref2 = this.region.ordered(), begin = _ref2.begin, end = _ref2.end;
      this.buffer.deleteLines(begin.row, end.row);
      this.toPoint();
      this.point.moveTo(begin);
      return this.point.round();
    };

    Cursor.prototype.text = function() {
      this.toRegion();
      return this.region.text();
    };

    Cursor.prototype.indent = function(tab) {
      if (this.point || this.region.isEmpty()) {
        return this.insert(tab);
      } else {
        return this.region.indent(tab);
      }
    };

    Cursor.prototype.outdent = function(tab) {
      this.toRegion();
      return this.region.outdent(tab);
    };

    Cursor.prototype.pageUp = function() {
      return this.toPoint();
    };

    Cursor.prototype.pageDown = function() {
      return this.toPoint();
    };

    return Cursor;

  })(EventEmitter);

  pointMethods = ["moveToLineBegin", "moveToLineEnd", "moveToPrevWord", "moveToNextWord", "moveToDocBegin", "moveToDocEnd", "moveLeft", "moveRight", "moveUp", "moveDown"];

  _fn = function(method) {
    Cursor.prototype[method] = function() {
      this.toPoint();
      this.point[method]();
      return this.point.round();
    };
    return Cursor.prototype[method.replace(/^move/, "select")] = function() {
      this.toRegion();
      this.region.end[method]();
      return this.region.end.round();
    };
  };
  for (_i = 0, _len = pointMethods.length; _i < _len; _i++) {
    method = pointMethods[_i];
    _fn(method);
  }

  regionMethods = ["shiftLinesDown", "shiftLinesUp"];

  _fn2 = function(method) {
    return Cursor.prototype[method] = function() {
      this.toRegion();
      return this.region[method]();
    };
  };
  for (_j = 0, _len2 = regionMethods.length; _j < _len2; _j++) {
    method = regionMethods[_j];
    _fn2(method);
  }

  module.exports = {
    Cursor: Cursor
  };

}).call(this);
