(function() {
  var Buffer, Cursor, Editor, EventEmitter, InsertMode, Layout, Point, Region, View, color, fractus, keyboard, _ref,
    __hasProp = Object.prototype.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; };

  EventEmitter = require('events').EventEmitter;

  color = require('stratus-color');

  keyboard = require('stratus-keyboard');

  _ref = require('./buffer'), Buffer = _ref.Buffer, Point = _ref.Point, Region = _ref.Region;

  Cursor = require('./cursor').Cursor;

  View = require('./view').View;

  Layout = require('./layout').Layout;

  InsertMode = require('./modes/insert_mode');

  module.exports = fractus = function($el, options) {
    return new Editor($el, options || {});
  };

  fractus.InsertMode = InsertMode;

  fractus.current = null;

  fractus.textarea = function($textarea, options) {
    var $el, ed, height, name, width;
    width = $textarea.width();
    height = $textarea.height();
    name = $textarea.attr("name");
    $el = $("<div/>");
    $el.css({
      width: width,
      height: height
    });
    $textarea.replaceWith($el);
    ed = new Editor($el, options || {});
    ed.view.input.$el.attr({
      name: name
    });
    return ed;
  };

  fractus.Editor = Editor = (function(_super) {

    __extends(Editor, _super);

    function Editor($el, options) {
      var mode, scrollwheel, syntax, text,
        _this = this;
      text = options.text, syntax = options.syntax, this.tab = options.tab, mode = options.mode, scrollwheel = options.scrollwheel, this.wrap = options.wrap;
      if (text == null) text = "";
      if (scrollwheel == null) scrollwheel = 4;
      if (this.tab == null) {
        this.tab = (syntax != null ? syntax.tab : void 0) || "    ";
      }
      this.setMode(mode || InsertMode);
      this.buffer = new Buffer(text, function() {
        return _this.cursor.serialize();
      }, function(data) {
        return _this.cursor.deserialize(data);
      });
      this.cursor = new Cursor(this.buffer, 0, 0);
      this.layout = new Layout(this.buffer, View.sizeOf, null, null, this.syntax);
      this.view = new View($el, this.buffer, this.layout, this.cursor, {
        scrollwheel: scrollwheel
      });
      this.view.on("focus", function() {
        fractus.current = _this;
        return _this.emit("focus");
      });
      this.view.on("blur", function() {
        return _this.emit("blur");
      });
      fractus.current = this;
      if (options.syntax) this.setSyntax(options.syntax);
    }

    Editor.prototype.setMode = function(mode) {
      this.mode = mode;
    };

    Editor.prototype.setSyntax = function(syntax) {
      this.syntax = syntax;
      this.layout.syntax = this.syntax.name;
      if (this.syntax.name === "Text") this.layout.syntax = null;
      return this.layout.onReset();
    };

    Editor.prototype.text = function() {
      return this.buffer.text();
    };

    Editor.prototype.focus = function() {
      return this.view.focus();
    };

    Editor.prototype.refresh = function() {
      return this.layout.scrollTo(this.layout.topRow, true);
    };

    Editor.prototype.resize = function() {
      this.view.onResize();
      return this.layout.scrollTo(this.layout.topRow, true);
    };

    Editor.prototype.emphasize = function(region, cssClass) {
      var coords;
      coords = this.layout.regionToCoords(region);
      $("<div></div>", {
        "class": "hi-emphasize " + cssClass
      }).appendTo(this.view.$scrollView).css(coords);
    };

    return Editor;

  })(EventEmitter);

  keyboard("fractus", function(key) {
    var action, edt, mode, val;
    edt = fractus.current;
    mode = edt.mode;
    if (typeof mode === "function") {
      val = mode(edt, key);
    } else if (action = mode[key]) {
      action(edt);
      val = false;
    } else {
      val = typeof mode["otherwise"] === "function" ? mode["otherwise"](edt, key) : void 0;
    }
    return val;
  });

}).call(this);
