(function() {
  var Buffer, EventEmitter, Layout, Point, Region, color, _, _ref,
    __hasProp = Object.prototype.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; };

  EventEmitter = require('events').EventEmitter;

  _ref = require('./buffer'), Buffer = _ref.Buffer, Point = _ref.Point, Region = _ref.Region;

  _ = require('underscore');

  color = require('stratus-color');

  module.exports.Layout = Layout = (function(_super) {

    __extends(Layout, _super);

    function Layout(buffer, sizeOf, height, width, syntax, uniform) {
      var _this = this;
      this.buffer = buffer;
      this.sizeOf = sizeOf;
      this.height = height;
      this.width = width;
      this.syntax = syntax != null ? syntax : null;
      this.uniform = uniform != null ? uniform : false;
      this.buffer.on("reset", function() {
        return _this.onReset();
      });
      this.buffer.on("line:change", function(data) {
        return _this.onLineChange(data);
      });
      this.buffer.on("line:insert", function(data) {
        return _this.onLineInsert(data);
      });
      this.buffer.on("line:delete", function(data) {
        return _this.onLineDelete(data);
      });
      if (this.syntax === "Text") this.syntax = null;
      this.hardTab = 8;
    }

    Layout.prototype.ready = function() {
      return this.onReset();
    };

    Layout.prototype.scrollTo = function(row, force) {
      var max, visibleRows;
      if (force == null) force = false;
      if (!force && this.scrollLock) return false;
      if (!force && row === this.topRow) return false;
      visibleRows = this.visibleRows();
      if (row instanceof Point) row = row.row;
      if (row < 0) row = 0;
      if (row > (max = this.buffer.lineCount() - visibleRows + 1)) row = max;
      if (visibleRows >= this.buffer.lineCount()) row = 0;
      if (!force && row === this.topRow) return false;
      this.topRow = row;
      this._renderVisible();
      this.emit("scrollY", this.scrollTop(row));
      this.renumber();
      return true;
    };

    Layout.prototype.scrollTop = function(row) {
      return row * this.sizeOf().height;
    };

    Layout.prototype.scrollBottom = function(row) {
      return this.scrollTo(row - this.visibleRows() + 1);
    };

    Layout.prototype.lockScroll = function() {
      return this.scrollLock = true;
    };

    Layout.prototype.unlockScroll = function() {
      return this.scrollLock = false;
    };

    Layout.prototype.totalHeight = function() {
      return (this.buffer.lineCount() + 1) * this.sizeOf().height;
    };

    Layout.prototype.lineHeight = function(row) {
      return this.sizeOf().height;
    };

    Layout.prototype.setWidth = function(width) {
      this.width = width;
    };

    Layout.prototype.setHeight = function(height) {
      this.height = height;
    };

    Layout.prototype.visibleRows = function() {
      return Math.floor(this.height / this.sizeOf().height);
    };

    Layout.prototype.isVisible = function(row) {
      return (this.topRow <= row && row <= this.topRow + this.visibleRows());
    };

    Layout.prototype.renderRow = function(row) {
      var line, lineHtml, tokens;
      if (!(row < this.buffer.lineCount())) return "";
      if (lineHtml = this.lineCache[row]) return lineHtml;
      if (this.syntax) {
        tokens = this.tokenize(row);
        this.lineCache[row] = "<li>" + (color.html(tokens)) + "</li>";
      } else {
        line = this.buffer.text(row);
        if (line != null) {
          this.lineCache[row] = "<li>" + (color.escape(line)) + "</li>";
        }
      }
      return this.lineCache[row];
    };

    Layout.prototype.tokenize = function(row) {
      var lineText, lineTokens, oldState, stack, tokens, _ref2, _ref3;
      if (lineTokens = this.tokenCache[row]) return lineTokens;
      lineText = "\n" + (this.buffer.text(row)) + "\n";
      if (!this.contexts[row]) this.tokenize(row - 1);
      oldState = ((_ref2 = this.contexts[row + 1]) != null ? _ref2.slice() : void 0) || [];
      _ref3 = color.tokenize(lineText, this.contexts[row].slice()), tokens = _ref3.tokens, stack = _ref3.stack;
      this.contexts[row + 1] = stack;
      if (_.last(stack) !== _.last(oldState)) {
        this.lineCache[row + 1] = null;
        this.tokenCache[row + 1] = null;
      }
      return this.tokenCache[row] = tokens;
    };

    Layout.prototype.onLineChange = function(data) {
      var newState, oldState, row, text;
      row = data.row, text = data.text;
      this.lineCache[row] = null;
      this.tokenCache[row] = null;
      if (this.syntax) oldState = this.contexts[row + 1].slice();
      if (this.isVisible(row)) {
        this.emit("render:line:change", {
          row: row,
          html: this.renderRow(row)
        });
      } else {
        this.lineCache[row + 1] = null;
        this.tokenCache[row + 1] = null;
      }
      if (this.syntax) {
        newState = this.contexts[row + 1];
        if (_.last(newState) !== _.last(oldState)) return this._updateRow(row + 1);
      }
    };

    Layout.prototype.onLineInsert = function(data) {
      var newState, oldState, row, text;
      this.onRowCountChange();
      row = data.row, text = data.text;
      this.lineCache.splice(row, 0, null);
      this.tokenCache.splice(row, 0, null);
      if (this.syntax) {
        oldState = this.contexts[row].slice();
        this.contexts.splice(row, 0, oldState.slice());
      }
      if (this.isVisible(row)) {
        this.emit("render:line:insert", {
          row: row,
          html: this.renderRow(row)
        });
      }
      if (this.syntax) {
        newState = this.contexts[row + 1];
        if (_.last(newState) !== _.last(oldState)) return this._updateRow(row + 1);
      }
    };

    Layout.prototype.onLineDelete = function(data) {
      var newState, nextRow, oldState, row, text;
      this.onRowCountChange();
      row = data.row, text = data.text;
      this.lineCache.splice(row, 1);
      this.tokenCache.splice(row, 1);
      if (this.syntax) {
        oldState = this.contexts[row].slice();
        this.contexts.splice(row + 1, 1);
        this.contexts[0] = [this.syntax];
      }
      if (this.isVisible(row)) {
        nextRow = this.topRow + this.visibleRows() - 1;
        this.emit("render:line:delete", {
          row: row,
          html: this.renderRow(nextRow)
        });
      }
      if (this.syntax) {
        newState = this.contexts[row];
        return this._updateRow(row);
      }
    };

    Layout.prototype.onReset = function() {
      this.onRowCountChange();
      this.tokenCache = [];
      this.lineCache = [];
      this.contexts = [[this.syntax]];
      this.scrollTo(0, 0);
      return this._renderVisible();
    };

    Layout.prototype._updateRow = function(row) {
      if (row === this.buffer.lineCount()) return;
      return this.onLineChange({
        row: row,
        text: this.buffer.text(row)
      });
    };

    Layout.prototype._renderVisible = function() {
      var topRow, visibleLines, _i, _ref2, _results,
        _this = this;
      topRow = this.topRow;
      visibleLines = (function() {
        _results = [];
        for (var _i = topRow, _ref2 = topRow + this.visibleRows() + 4; topRow <= _ref2 ? _i <= _ref2 : _i >= _ref2; topRow <= _ref2 ? _i++ : _i--){ _results.push(_i); }
        return _results;
      }).apply(this);
      visibleLines = _.map(visibleLines, function(r) {
        return _this.renderRow(r);
      });
      return this.emit("render", visibleLines.join(""));
    };

    Layout.prototype.pointToCoords = function(point) {
      var col, left, line, row, top;
      row = point.row, col = point.col;
      top = this.sizeOf().height * row;
      if (this.uniform) {
        left = this.widthOf(row, 0, col);
      } else {
        line = this.buffer.text(row);
        if (!line) {
          left = this.sizeOf().width * col;
        } else {
          left = this.widthOf(row, 0, col);
        }
      }
      left += this.gutterWidth;
      return [top, left];
    };

    Layout.prototype.regionToCoords = function(region) {
      var begin, beginLeft, beginTop, end, endLeft, endTop, _ref2, _ref3, _ref4;
      _ref2 = region.ordered(), begin = _ref2.begin, end = _ref2.end;
      _ref3 = this.pointToCoords(begin), beginTop = _ref3[0], beginLeft = _ref3[1];
      _ref4 = this.pointToCoords(end), endTop = _ref4[0], endLeft = _ref4[1];
      return {
        top: beginTop,
        left: beginLeft,
        width: endLeft - beginLeft,
        height: this.lineHeight(begin.row)
      };
    };

    Layout.prototype.coordsToPoint = function(top, left) {
      var carat, char, col, colCarat, i, line, row, tabCols, width, _len;
      row = Math.floor(top / this.sizeOf().height + this.topRow);
      if (this.uniform) {
        col = Math.round(left / this.sizeOf().width);
      } else if (!(line = this.buffer.text(row)) || !line.length) {
        col = 0;
      } else {
        carat = 0;
        colCarat = 0;
        for (i = 0, _len = line.length; i < _len; i++) {
          char = line[i];
          if (char === "\t") {
            tabCols = this.tabWidth(colCarat);
            width = tabCols * this.sizeOf().width;
            colCarat += tabCols;
          } else {
            width = this.sizeOf(char).width;
            colCarat++;
          }
          carat += width;
          if (carat === left) {
            col = i + 1;
            break;
          } else if (carat > left) {
            if (left > carat - (width / 2)) {
              col = i + 1;
            } else {
              col = i;
            }
            break;
          }
        }
        if (col == null) col = line.length;
      }
      return this.buffer.point(row, col);
    };

    Layout.prototype.widthOf = function(row, colBegin, colEnd) {
      var carat, char, i, line, tabCols, width, _len;
      if (this.uniform) {
        return this.sizeOf().width * col;
      } else {
        line = this.buffer.text(row).substr(colBegin, colEnd);
        width = 0;
        carat = colBegin;
        for (i = 0, _len = line.length; i < _len; i++) {
          char = line[i];
          if (char === "\t") {
            tabCols = this.tabWidth(carat);
            width += tabCols * this.sizeOf().width;
            carat += tabCols;
          } else {
            width += this.sizeOf(char).width;
            carat++;
          }
        }
        return width;
      }
    };

    Layout.prototype.tabWidth = function(col) {
      return this.hardTab - (col % this.hardTab);
    };

    Layout.prototype.isRowTop = function(top) {
      return Math.floor(top / this.sizeOf().height) < this.buffer.lineCount();
    };

    Layout.prototype.isTooFarUp = function(point) {
      return point.row < this.topRow;
    };

    Layout.prototype.isTooFarDown = function(point) {
      return point.row + 1 > this.topRow + this.visibleRows();
    };

    Layout.prototype.gutterChars = function() {
      return ("" + this.buffer.lineCount()).length;
    };

    Layout.prototype.updateGutterWidth = function() {
      var gw;
      if (this.gutterWidth === (gw = this.calcGutterWidth())) return;
      return this.emit("gutter:resize", this.gutterWidth = gw);
    };

    Layout.prototype.calcGutterWidth = function() {
      return this.sizeOf().width * (this.gutterChars() + 1);
    };

    Layout.prototype.onRowCountChange = function() {
      this.updateGutterWidth();
      this.renumber();
      return this.emit("change:height", this.totalHeight());
    };

    Layout.prototype.renumber = function() {
      var end, max;
      end = this.topRow + this.visibleRows();
      if (end > (max = this.buffer.lineCount() - 1)) end = max;
      return this.emit("gutter:renumber", {
        begin: this.topRow,
        end: end
      });
    };

    return Layout;

  })(EventEmitter);

}).call(this);
